@@ -205,6 +205,7 @@ struct drm_agp_mem;
 struct drm_map;
 struct firegl_pcie_mem;
 struct firegl_pcie;
+struct proc_dir_entry;
 
 #if !defined(ATI_API_CALL)
 #define ATI_API_CALL __attribute__((regparm(0)))
@@ -221,6 +222,7 @@ typedef unsigned short __ke_u16;
 typedef unsigned int __ke_u32;
 typedef unsigned long long __ke_u64;
 typedef unsigned long long __ke_dma_addr_t;
+typedef long long __ke_loff_t;
 
 // note: assigning uniqe types to originally non interchangeable types
 typedef struct { int uniqe1; } __ke_wait_queue_head_t;
@@ -229,13 +231,16 @@ typedef struct { int uniqe3; } __ke_sigset_t;
 typedef struct { int uniqe4; } __ke_pci_dev_t;
 typedef struct { int uniqe5; } __ke_priv_device_t;
 typedef struct { int uniqe6; } __ke_pci_bus_t;
+typedef struct { int uniqe7; } __ke_file_operations_t;
+typedef struct { int uniqe8; } __ke_poll_table;
 
 typedef	int (*__ke_read_proc_t)(
     char* page, char** start, __ke_off_t off, int count, int* eof, void* data);
 
 typedef struct {
-    const char*         name;
-    __ke_read_proc_t    f;
+    const char*             name;
+    __ke_read_proc_t        f;
+    __ke_file_operations_t* fops;
 } __ke_proc_list_t;
 
 extern __ke_proc_list_t firegl_proc_list[];
@@ -264,7 +269,6 @@ typedef struct __ke_pci_device_info
 
 extern int ATI_API_CALL drm_name_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
 extern int ATI_API_CALL firegl_bios_version(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
-extern int ATI_API_CALL firegl_interrupt_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
 extern int ATI_API_CALL drm_mem_info(char* buf, char** start, __ke_off_t offset, int len, int *eof, void *data);
 extern int ATI_API_CALL drm_mem_info1(char* buf, char** start, __ke_off_t offset, int len, int *eof, void *data);
 extern int ATI_API_CALL drm_vm_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
@@ -275,6 +279,19 @@ extern int ATI_API_CALL firegl_umm_info(char* buf, char** start, __ke_off_t offs
 extern int ATI_API_CALL drm_bq_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
 extern int ATI_API_CALL firegl_debug_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
 #endif
+extern int ATI_API_CALL firegl_interrupt_open(struct inode* inode, struct file* file);
+extern int ATI_API_CALL firegl_interrupt_release(struct inode* inode, struct file* file);
+extern unsigned int ATI_API_CALL firegl_interrupt_read(
+                                    struct file *user_file, 
+                                    char *user_buf, 
+                                    __ke_size_t user_buf_size, 
+                                    __ke_loff_t *user_file_pos);
+extern unsigned int ATI_API_CALL firegl_interrupt_poll(struct file *user_file, __ke_poll_table *pt);
+extern int ATI_API_CALL firegl_interrupt_write(
+                                    struct file *user_file, 
+                                    char *user_buf, 
+                                    __ke_size_t user_buf_size, 
+                                    __ke_loff_t *user_file_pos);
 
 /*****************************************************************************/
 
@@ -323,11 +340,13 @@ extern void ATI_API_CALL __ke_free_wait_queue_head_struct(__ke_wait_queue_head_t
 extern __ke_wait_queue_t* ATI_API_CALL __ke_alloc_wait_queue_struct(void);
 extern void ATI_API_CALL __ke_free_wait_queue_struct(__ke_wait_queue_t* queue);
 
+extern void ATI_API_CALL __ke_wait_event_interruptible(__ke_wait_queue_head_t* queue_head, int condition);
 extern void ATI_API_CALL __ke_wake_up_interruptible(__ke_wait_queue_head_t* queue_head);
+extern void ATI_API_CALL __ke_poll_wait(struct file* filp, __ke_wait_queue_head_t* queue_head, __ke_poll_table* pt);
+    
 extern void ATI_API_CALL __ke_add_wait_queue(__ke_wait_queue_head_t* queue_head, __ke_wait_queue_t* entry);
 extern void ATI_API_CALL __ke_remove_wait_queue(__ke_wait_queue_head_t* queue_head, __ke_wait_queue_t* entry);
 extern void ATI_API_CALL __ke_init_waitqueue_head(__ke_wait_queue_head_t* queue_head);
-extern void ATI_API_CALL __ke_wait_event_interruptible(__ke_wait_queue_head_t* queue_head, int condition);
 
 extern void ATI_API_CALL __ke_schedule(void);
 extern int ATI_API_CALL __ke_signal_pending(void);
@@ -354,6 +373,8 @@ extern unsigned long long ATI_API_CALL __ke_cpu_to_le64(unsigned long long _u);
 extern __ke_dev_t ATI_API_CALL __ke_getdevice(__ke_device_t *dev);
 extern const char* ATI_API_CALL __ke_module_parm(void);
 extern int ATI_API_CALL __ke_inode_rdev_minor(struct inode* inode);
+extern void* ATI_API_CALL __ke_get_proc_dir_entry_priv(struct proc_dir_entry* pde);
+extern void* ATI_API_CALL __ke_get_inode_priv(struct inode* inode);
 extern void* ATI_API_CALL __ke_get_file_priv(struct file* filp);
 extern void ATI_API_CALL __ke_set_file_priv(struct file* filp, void* private_data);
 extern int ATI_API_CALL __ke_file_excl_open(struct file* filp);
@@ -446,9 +467,12 @@ extern int ATI_API_CALL __ke_copy_to_user(void* to, const void* from, __ke_size_
 extern int ATI_API_CALL __ke_verify_area(int type, const void * addr, unsigned long size);
 
 extern void* ATI_API_CALL __ke_malloc(__ke_size_t size);
+extern void* ATI_API_CALL __ke_malloc_atomic(__ke_size_t size);
+extern void ATI_API_CALL __ke_free(void* p);
 extern void ATI_API_CALL __ke_free_s(void* p, __ke_size_t size);
 extern void* ATI_API_CALL __ke_vmalloc(__ke_size_t size);
 extern void* ATI_API_CALL __ke_vmalloc_32(__ke_size_t size);
+extern void* ATI_API_CALL __ke_vmalloc_atomic(__ke_size_t size);
 extern void ATI_API_CALL __ke_vfree(void* p);
 extern void* ATI_API_CALL __ke_get_free_page(void);
 extern void* ATI_API_CALL __ke_get_free_pages(int order);
@@ -482,6 +506,7 @@ extern char* ATI_API_CALL __ke_strcpy(char* d, const char* s);
 extern char* ATI_API_CALL __ke_strncpy(char* d, const char* s, __ke_size_t count);
 extern int ATI_API_CALL __ke_strcmp(const char *s1, const char *s2);
 extern int ATI_API_CALL __ke_strncmp(const char* str1, const char* str2, __ke_size_t count);
+extern char* ATI_API_CALL __ke_strchr(const char *s, int c);
 extern int ATI_API_CALL __ke_sprintf(char* buf, const char* fmt, ...);
 extern int ATI_API_CALL __ke_snprintf(char* buf, size_t size, const char* fmt, ...);
 extern int ATI_API_CALL __ke_vm_test_and_clear_dirty(struct mm_struct* mm, unsigned long virtual_addr);
